import textwrap
import unittest

from brew.yeasts import Yeast

from fixtures import yeast


class TestYeasts(unittest.TestCase):

    def setUp(self):

        # Define Yeasts
        self.yeast = yeast

    def test_str(self):
        out = str(self.yeast)
        self.assertEquals(out, 'Wyeast 1056, attenuation 0.75%')

    def test_repr(self):
        out = repr(self.yeast)
        self.assertEquals(out, "Yeast('Wyeast 1056', percent_attenuation=0.75)")

    def test_to_dict(self):
        out = self.yeast.to_dict()
        expected = {'name': 'Wyeast 1056',
                    'data': {
                        'percent_attenuation': 0.75,
                    },
                    }
        self.assertEquals(out, expected)

    def test_to_json(self):
        out = self.yeast.to_json()
        expected = '{"data": {"percent_attenuation": 0.75}, "name": "Wyeast 1056"}'  # nopep8
        self.assertEquals(out, expected)

    def test_validate(self):
        data = self.yeast.to_dict()
        Yeast.validate(data)

    def test_format(self):
        out = self.yeast.format()
        msg = textwrap.dedent("""\
                Wyeast 1056 Yeast
                -----------------------------------
                Attenuation:  0.75 %""")
        self.assertEquals(out, msg)



import unittest

from brew.constants import IMPERIAL_UNITS
from brew.constants import SI_UNITS
from brew.hops import HopAddition
from brew.utilities.hops import HopsUtilization
from brew.utilities.hops import HopsUtilizationGlennTinseth
from brew.utilities.hops import HopsUtilizationJackieRager
from brew.utilities.sugar import plato_to_sg
from fixtures import centennial


class TestHopsUtilization(unittest.TestCase):

    def setUp(self):
        self.utilization_cls = HopsUtilization
        self.hop = centennial
        self.addition_kwargs = [
            {
                'boil_time': 60.0,
                'weight': 0.57,
            }
        ]

        # Additions
        self.plato = 14.0
        self.sg = plato_to_sg(self.plato)
        self.final_volume = 5.0
        self.boil_time = 60.0
        self.hop_addition = HopAddition(self.hop,
                                        boil_time=self.boil_time,
                                        weight=0.57,
                                        utilization_cls=self.utilization_cls)

    def test_get_ibus_raises(self):
        with self.assertRaises(NotImplementedError):
            self.hop_addition.get_ibus(self.sg, self.final_volume)

    def test_get_percent_utilization_raises(self):
        with self.assertRaises(NotImplementedError):
            self.hop_addition.utilization_cls.get_percent_utilization(
                self.sg, self.final_volume)

    def test_change_units(self):
        self.assertEquals(self.hop_addition.utilization_cls.units,
                          IMPERIAL_UNITS)
        util = self.hop_addition.utilization_cls.change_units()
        self.assertEquals(util.units, SI_UNITS)
        util = util.change_units()
        self.assertEquals(util.units, IMPERIAL_UNITS)


class TestHopsUtilizationJackieRager(unittest.TestCase):

    def setUp(self):
        self.utilization_cls = HopsUtilizationJackieRager
        self.hop = centennial
        self.addition_kwargs = [
            {
                'boil_time': 60.0,
                'weight': 0.57,
            }
        ]

        # Additions
        self.plato = 14.0
        self.sg = plato_to_sg(self.plato)
        self.final_volume = 5.0
        self.boil_time = 60.0
        self.hop_addition = HopAddition(self.hop,
                                        boil_time=self.boil_time,
                                        weight=0.57,
                                        utilization_cls=self.utilization_cls)

    def test_str(self):
        self.assertEquals(str(self.hop_addition.utilization_cls),
                          "Jackie Rager")

    def test_get_c_gravity(self):
        out = self.hop_addition.utilization_cls.get_c_gravity(self.sg)
        self.assertEquals(round(out, 3), 1.034)
        out = self.hop_addition.utilization_cls.get_c_gravity(1.050)
        self.assertEquals(round(out, 3), 1.000)
        out = self.hop_addition.utilization_cls.get_c_gravity(1.010)
        self.assertEquals(round(out, 3), 1.000)

    def test_get_ibus(self):
        ibu = self.hop_addition.get_ibus(self.sg,
                                         self.final_volume)
        self.assertEquals(round(ibu, 2), 39.18)

    def test_get_percent_utilization(self):
        utilization = self.hop_addition.utilization_cls.get_percent_utilization(  # nopep8
                self.sg, self.boil_time)
        self.assertEquals(round(utilization * 100, 2), 29.80)

    def test_get_utilization_table(self):
        gravity_list = list(range(1030, 1140, 10))
        boil_time_list = list(range(0, 60, 3)) + list(range(60, 130, 10))
        table = self.utilization_cls.get_utilization_table(
            gravity_list,
            boil_time_list)
        self.assertEquals(table[0][0], 0.051)
        self.assertEquals(table[13][5], 0.205)
        self.assertEquals(table[26][10], 0.228)

    def test_format_utilization_table(self):
        out = self.utilization_cls.format_utilization_table()
        expected = ("""\
            Percent Alpha Acid Utilization - Boil Time vs Wort Original Gravity
            ===================================================================
       1.030   1.040   1.050   1.060   1.070   1.080   1.090   1.100   1.110   1.120   1.130
--------------------------------------------------------------------------------------------
   0   0.051   0.051   0.051   0.049   0.047   0.045   0.043   0.041   0.039   0.038   0.037
   3   0.054   0.054   0.054   0.052   0.049   0.047   0.045   0.044   0.042   0.040   0.039
   6   0.059   0.059   0.059   0.056   0.053   0.051   0.049   0.047   0.045   0.044   0.042
   9   0.065   0.065   0.065   0.062   0.059   0.056   0.054   0.052   0.050   0.048   0.046
  12   0.072   0.072   0.072   0.069   0.066   0.063   0.060   0.058   0.056   0.054   0.052
  15   0.082   0.082   0.082   0.078   0.075   0.072   0.069   0.066   0.063   0.061   0.059
  18   0.095   0.095   0.095   0.090   0.086   0.082   0.079   0.076   0.073   0.070   0.068
  21   0.110   0.110   0.110   0.105   0.100   0.096   0.092   0.088   0.085   0.082   0.079
  24   0.128   0.128   0.128   0.122   0.117   0.112   0.107   0.103   0.099   0.095   0.092
  27   0.149   0.149   0.149   0.142   0.135   0.129   0.124   0.119   0.115   0.110   0.106
  30   0.171   0.171   0.171   0.163   0.156   0.149   0.143   0.137   0.132   0.127   0.122
  33   0.194   0.194   0.194   0.185   0.176   0.169   0.162   0.155   0.149   0.144   0.138
  36   0.216   0.216   0.216   0.206   0.196   0.188   0.180   0.173   0.166   0.160   0.154
  39   0.236   0.236   0.236   0.225   0.215   0.205   0.197   0.189   0.182   0.175   0.169
  42   0.254   0.254   0.254   0.242   0.231   0.221   0.212   0.203   0.195   0.188   0.181
  45   0.269   0.269   0.269   0.256   0.245   0.234   0.224   0.215   0.207   0.199   0.192
  48   0.281   0.281   0.281   0.268   0.256   0.245   0.234   0.225   0.216   0.208   0.201
  51   0.291   0.291   0.291   0.277   0.264   0.253   0.242   0.233   0.224   0.215   0.208
  54   0.298   0.298   0.298   0.284   0.271   0.259   0.249   0.239   0.229   0.221   0.213
  57   0.304   0.304   0.304   0.290   0.276   0.264   0.253   0.243   0.234   0.225   0.217
  60   0.308   0.308   0.308   0.294   0.280   0.268   0.257   0.247   0.237   0.228   0.220
  70   0.316   0.316   0.316   0.301   0.287   0.275   0.263   0.253   0.243   0.234   0.226
  80   0.318   0.318   0.318   0.303   0.289   0.277   0.265   0.255   0.245   0.236   0.227
  90   0.319   0.319   0.319   0.304   0.290   0.278   0.266   0.255   0.246   0.236   0.228
 100   0.320   0.320   0.320   0.304   0.290   0.278   0.266   0.256   0.246   0.237   0.228
 110   0.320   0.320   0.320   0.304   0.291   0.278   0.266   0.256   0.246   0.237   0.228
 120   0.320   0.320   0.320   0.304   0.291   0.278   0.266   0.256   0.246   0.237   0.228""")  # nopep8
        self.assertEquals(out, expected)


class TestHopsUtilizationGlennTinseth(unittest.TestCase):

    def setUp(self):
        self.utilization_cls = HopsUtilizationGlennTinseth
        self.hop = centennial
        self.addition_kwargs = [
            {
                'boil_time': 60.0,
                'weight': 0.57,
            }
        ]

        # Additions
        self.plato = 14.0
        self.sg = plato_to_sg(self.plato)
        self.final_volume = 5.0
        self.boil_time = 60.0
        self.hop_addition = HopAddition(self.hop,
                                        boil_time=self.boil_time,
                                        weight=0.57,
                                        utilization_cls=self.utilization_cls)

    def test_str(self):
        self.assertEquals(str(self.hop_addition.utilization_cls),
                          "Glenn Tinseth")

    def test_get_ibus(self):
        ibu = self.hop_addition.get_ibus(self.sg,
                                         self.final_volume)
        self.assertEquals(round(ibu, 2), 28.52)

    def test_get_bigness_factor(self):
        bf = self.hop_addition.utilization_cls.get_bigness_factor(self.sg)
        self.assertEquals(round(bf, 2), 0.99)

    def test_get_boil_time_factor(self):
        bf = self.hop_addition.utilization_cls.get_boil_time_factor(
            self.boil_time)
        self.assertEquals(round(bf, 2), 0.22)

    def test_get_percent_utilization(self):
        utilization = self.hop_addition.utilization_cls.get_percent_utilization(  # nopep8
                self.sg, self.boil_time)
        self.assertEquals(round(utilization * 100, 2), 21.69)



import unittest

from brew.constants import SI_UNITS
from brew.utilities.sugar import brix_to_plato
from brew.utilities.sugar import brix_to_sg
from brew.utilities.sugar import gu_to_sg
from brew.utilities.sugar import hydrometer_adjustment
from brew.utilities.sugar import plato_to_brix
from brew.utilities.sugar import plato_to_sg
from brew.utilities.sugar import refractometer_adjustment
from brew.utilities.sugar import sg_to_brix
from brew.utilities.sugar import sg_to_gu
from brew.utilities.sugar import sg_to_plato


class TestSugarUtilities(unittest.TestCase):

    def test_brix_to_plato(self):
        plato = brix_to_plato(22.0)
        self.assertEquals(round(plato, 3), 22.001)

    def test_brix_to_sg(self):
        sg = brix_to_sg(22.0)
        self.assertEquals(round(sg, 3), 1.092)

    def test_gu_to_sg(self):
        sg = gu_to_sg(57.0)
        self.assertEquals(round(sg, 3), 1.057)

    def test_plato_to_brix(self):
        brix = plato_to_brix(14.0)
        self.assertEquals(round(brix, 3), 14.002)

    def test_plato_to_sg(self):
        sg = plato_to_sg(14.0)
        self.assertEquals(round(sg, 3), 1.057)

    def test_sg_to_brix(self):
        brix = sg_to_brix(1.092)
        self.assertEquals(round(brix, 1), 22.0)

    def test_sg_to_brix_raises(self):
        with self.assertRaises(Exception):
            sg_to_brix(1.18)

    def test_sg_to_plato(self):
        plato = sg_to_plato(1.0570)
        self.assertEquals(round(plato, 2), 14.04)

    def test_sg_to_gu(self):
        gu = sg_to_gu(1.057)
        self.assertEquals(round(gu, 2), 57.0)

    def test_hydrometer_adjustment(self):
        sg = hydrometer_adjustment(1.050, 70.0)
        self.assertEquals(round(sg, 3), 1.051)

    def test_hydrometer_adjustment_no_adjustment(self):
        sg = hydrometer_adjustment(1.050, 59.0)
        self.assertEquals(round(sg, 3), 1.050)

    def test_hydromter_adjustment_si_units(self):
        sg = hydrometer_adjustment(1.050, 16.0, units=SI_UNITS)
        self.assertEquals(round(sg, 3), 1.050)

    def test_hydrometer_adjustment_raises_bad_units(self):
        with self.assertRaises(Exception):
            hydrometer_adjustment(1.050, 16.0, units='bad')

    def test_hydrometer_adjustment_raises_bad_temp(self):
        with self.assertRaises(Exception):
            hydrometer_adjustment(1.050, -1.0)

        with self.assertRaises(Exception):
            hydrometer_adjustment(1.050, 213.0)

        with self.assertRaises(Exception):
            hydrometer_adjustment(1.050, -1.0, units=SI_UNITS)

        with self.assertRaises(Exception):
            hydrometer_adjustment(1.050, 101.0, units=SI_UNITS)

    def test_refractometer_adjustment(self):
        fg = refractometer_adjustment(1.050, 1.011)
        self.assertEquals(round(fg, 3), 1.004)



import textwrap
import unittest

from brew.cli.abv import get_abv
from brew.cli.abv import get_parser
from brew.constants import IMPERIAL_UNITS
from brew.constants import SI_UNITS


class TestCliAbv(unittest.TestCase):

    def setUp(self):
        self.og = 1.057
        self.fg = 1.013

    def test_get_abv(self):
        abv = get_abv(self.og, self.fg)
        self.assertEquals(round(abv, 2), 5.78)

    def test_get_abv_no_og_raises(self):
        with self.assertRaises(Exception):
            get_abv(None, self.fg)

    def test_get_abv_no_fg_raises(self):
        with self.assertRaises(Exception):
            get_abv(self.og, None)

    def test_get_abv_og_fg_reversed_raises(self):
        with self.assertRaises(Exception):
            get_abv(self.fg, self.og)

    def test_get_abv_bad_units_raises(self):
        with self.assertRaises(Exception):
            get_abv(self.og, self.og, units='bad')

    def test_get_abv_units_imperial(self):
        abv = get_abv(self.og, self.fg, units=IMPERIAL_UNITS)
        self.assertEquals(round(abv, 2), 5.78)

    def test_get_abv_units_si(self):
        abv = get_abv(self.og, self.fg, units=SI_UNITS)
        self.assertEquals(round(abv, 2), 5.78)

    def test_get_abv_alternative(self):
        abv = get_abv(self.og, self.fg, alternative=True)
        self.assertEquals(round(abv, 2), 5.95)

    def test_get_abv_refractometer(self):
        abv = get_abv(self.og, self.fg, refractometer=True)
        self.assertEquals(round(abv, 2), 6.97)

    def test_get_abv_alternative_refractometer(self):
        abv = get_abv(self.og, self.fg, alternative=True, refractometer=True)
        self.assertEquals(round(abv, 2), 7.12)

    def test_get_abv_verbose(self):
        out = get_abv(self.og, self.fg, verbose=True)
        expected = textwrap.dedent("""\
            OG     : 1.057
            OG Adj : 1.057
            OG Temp: 59.00 F
            FG     : 1.013
            FG Adj : 1.013
            FG Temp: 59.00 F
            ABV    : 5.78 %""")
        self.assertEquals(out, expected)


class TestCliArgparser(unittest.TestCase):

    def setUp(self):
        self.parser = get_parser()

    def test_get_parser_required_og_and_fg(self):
        with self.assertRaises(SystemExit):
            self.parser.parse_args([])

    def test_get_parser_required_og(self):
        with self.assertRaises(SystemExit):
            self.parser.parse_args(['-f', '1.010'])

    def test_get_parser_required_fg(self):
        with self.assertRaises(SystemExit):
            self.parser.parse_args(['-o', '1.060'])

    def test_get_parser_og_and_fg(self):
        args = ['-o', '1.060', '-f', '1.010']
        out = self.parser.parse_args(args)
        expected = {
            'alternative': False,
            'fg': 1.01,
            'fg_temp': 59.0,
            'og': 1.06,
            'og_temp': 59.0,
            'refractometer': False,
            'units': 'imperial',
            'verbose': False,
        }
        self.assertEquals(out.__dict__, expected)

    def test_get_parser_update_temp(self):
        args = ['-o', '1.060', '-f', '1.010',
                '--og-temp', '61.0',
                '--fg-temp', '61.0']
        out = self.parser.parse_args(args)
        expected = {
            'alternative': False,
            'fg': 1.01,
            'fg_temp': 61.0,
            'og': 1.06,
            'og_temp': 61.0,
            'refractometer': False,
            'units': 'imperial',
            'verbose': False,
        }
        self.assertEquals(out.__dict__, expected)

    def test_get_parser_alternative(self):
        args = ['-o', '1.060', '-f', '1.010', '--alternative']
        out = self.parser.parse_args(args)
        expected = {
            'alternative': True,
            'fg': 1.01,
            'fg_temp': 59.0,
            'og': 1.06,
            'og_temp': 59.0,
            'refractometer': False,
            'units': 'imperial',
            'verbose': False,
        }
        self.assertEquals(out.__dict__, expected)

    def test_get_parser_refractometer(self):
        args = ['-o', '1.060', '-f', '1.010', '--refractometer']
        out = self.parser.parse_args(args)
        expected = {
            'alternative': False,
            'fg': 1.01,
            'fg_temp': 59.0,
            'og': 1.06,
            'og_temp': 59.0,
            'refractometer': True,
            'units': 'imperial',
            'verbose': False,
        }
        self.assertEquals(out.__dict__, expected)

    def test_get_parser_units(self):
        args = ['-o', '1.060', '-f', '1.010', '--units', 'si']
        out = self.parser.parse_args(args)
        expected = {
            'alternative': False,
            'fg': 1.01,
            'fg_temp': 59.0,
            'og': 1.06,
            'og_temp': 59.0,
            'refractometer': False,
            'units': 'si',
            'verbose': False,
        }
        self.assertEquals(out.__dict__, expected)

    def test_get_parser_verbose(self):
        args = ['-o', '1.060', '-f', '1.010', '-v']
        out = self.parser.parse_args(args)
        expected = {
            'alternative': False,
            'fg': 1.01,
            'fg_temp': 59.0,
            'og': 1.06,
            'og_temp': 59.0,
            'refractometer': False,
            'units': 'imperial',
            'verbose': True,
        }
        self.assertEquals(out.__dict__, expected)



from setuptools import find_packages
from setuptools import setup


VERSION = '0.0.4'

setup(
    name='brewday',
    version=VERSION,
    author='Chris Gilmer',
    author_email='chris.gilmer@gmail.com',
    maintainer='Chris Gilmer',
    maintainer_email='chris.gilmer@gmail.com',
    license="MIT",
    description='Brew Day Tools',
    url='https://github.com/chrisgilmerproj/brewday',
    download_url='https://github.com/chrisgilmerproj/brewday/tarball/{}'.format(VERSION),  # nopep8
    packages=find_packages(exclude=["*.tests",
                                    "*.tests.*",
                                    "tests.*",
                                    "tests"]),
    entry_points={
        'console_scripts': [
            'abv = brew.cli.abv:main',
            'sugar = brew.cli.sugar:main',
            'temp = brew.cli.temp:main',
            'yeast = brew.cli.yeast:main',
        ],
    },
    include_package_data=True,
    zip_safe=True,
    tests_require=[
        'nose==1.3.1',
        'pluggy==0.3.1',
        'py==1.4.31',
        'tox==2.3.1',
    ],
    keywords='brew brewing beer grain hops yeast alcohol',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



import unittest

from brew.utilities.malt import as_is_basis_to_dry_basis
from brew.utilities.malt import basis_to_hwe
from brew.utilities.malt import coarse_grind_to_fine_grind
from brew.utilities.malt import dry_basis_to_as_is_basis
from brew.utilities.malt import dry_to_liquid_malt_weight
from brew.utilities.malt import dry_malt_to_grain_weight
from brew.utilities.malt import fine_grind_to_coarse_grind
from brew.utilities.malt import grain_to_dry_malt_weight
from brew.utilities.malt import grain_to_liquid_malt_weight
from brew.utilities.malt import hwe_to_basis
from brew.utilities.malt import hwe_to_ppg
from brew.utilities.malt import liquid_malt_to_grain_weight
from brew.utilities.malt import liquid_malt_to_specialty_grain_weight
from brew.utilities.malt import liquid_to_dry_malt_weight
from brew.utilities.malt import plato_from_dry_basis
from brew.utilities.malt import ppg_to_hwe
from brew.utilities.malt import sg_from_dry_basis
from brew.utilities.malt import specialty_grain_to_liquid_malt_weight


class TestMaltUtilities(unittest.TestCase):

    def test_dry_to_liquid_malt_weight(self):
        out = dry_to_liquid_malt_weight(5.0)
        self.assertEquals(out, 6.25)

    def test_liquid_to_dry_malt_weight(self):
        out = liquid_to_dry_malt_weight(6.25)
        self.assertEquals(out, 5.0)

    def test_grain_to_liquid_malt_weight(self):
        out = grain_to_liquid_malt_weight(5.0)
        self.assertEquals(out, 3.75)

    def test_liquid_malt_to_grain_weight(self):
        out = liquid_malt_to_grain_weight(3.75)
        self.assertEquals(out, 5.0)

    def test_grain_to_dry_malt_weight(self):
        out = grain_to_dry_malt_weight(5.0)
        self.assertEquals(out, 3.0)

    def test_dry_malt_to_grain_weight(self):
        out = dry_malt_to_grain_weight(3.75)
        self.assertEquals(out, 6.25)

    def test_specialty_grain_to_liquid_malt_weight(self):
        out = specialty_grain_to_liquid_malt_weight(5.0)
        self.assertEquals(out, 4.45)

    def test_liquid_malt_to_specialty_grain_weight(self):
        out = liquid_malt_to_specialty_grain_weight(4.45)
        self.assertEquals(out, 5.0)

    def test_fine_grind_to_coarse_grind(self):
        cg = fine_grind_to_coarse_grind(0.81)
        self.assertEquals(round(cg, 2), 0.79)

    def test_coarse_grind_to_fine_grind(self):
        fg = coarse_grind_to_fine_grind(0.79)
        self.assertEquals(round(fg, 2), 0.81)

    def test_dry_basis_to_as_is_basis(self):
        asb = dry_basis_to_as_is_basis(0.40)
        self.assertEquals(round(asb, 2), 0.38)

    def test_as_is_basis_to_dry_basis(self):
        db = as_is_basis_to_dry_basis(0.38)
        self.assertEquals(round(db, 2), 0.40)

    def test_sg_from_dry_basis(self):
        sg = sg_from_dry_basis(0.80)
        self.assertEquals(round(sg, 3), 1.032)

    def test_plato_from_dry_basis(self):
        plato = plato_from_dry_basis(0.80)
        self.assertEquals(round(plato, 2), 7.86)

    def test_basis_to_hwe(self):
        hwe = basis_to_hwe(0.8)
        self.assertEquals(round(hwe, 2), 308.8)

    def test_hwe_to_basis(self):
        basis = hwe_to_basis(308.8)
        self.assertEquals(round(basis, 3), 0.8)

    def test_ppg_to_hwe(self):
        hwe = ppg_to_hwe(37)
        self.assertEqual(round(hwe, 2), 308.78)

    def test_hwe_to_ppg(self):
        ppg = hwe_to_ppg(308.78)
        self.assertEqual(round(ppg, 2), 37)



import textwrap
import unittest

from brew.constants import GRAIN_TYPE_DME
from brew.constants import GRAIN_TYPE_LME
from brew.constants import IMPERIAL_UNITS
from brew.constants import LITER_PER_GAL
from brew.constants import SI_UNITS
from brew.constants import SUCROSE_PLATO
from brew.grains import GrainAddition
from brew.recipes import Recipe
from fixtures import grain_additions
from fixtures import hop_additions
from fixtures import pale
from fixtures import recipe
from fixtures import yeast


class TestRecipeSIUnits(unittest.TestCase):

    def setUp(self):
        # Define Grains
        self.grain_additions = [ga.change_units() for ga in
                                grain_additions]

        # Define Hops
        self.hop_additions = [ha.change_units() for ha in
                              hop_additions]

        # Define Yeast
        self.yeast = yeast

        # Define Recipes
        self.recipe = recipe.change_units()
        self.assertEquals(self.recipe.units, SI_UNITS)

    def test_change_units(self):
        self.assertEquals(self.recipe.units, SI_UNITS)
        self.assertEquals(round(self.recipe.start_volume, 2), 26.50)
        self.assertEquals(round(self.recipe.final_volume, 2), 18.93)
        self.assertEquals(round(self.recipe.grain_additions[0].weight, 2), 6.33)  # nopep8
        self.assertEquals(round(self.recipe.grain_additions[1].weight, 2), 0.35)  # nopep8
        self.assertEquals(round(self.recipe.hop_additions[0].weight, 2), 16159.21)  # nopep8
        self.assertEquals(round(self.recipe.hop_additions[1].weight, 2), 21545.62)  # nopep8
        for grain_add in self.recipe.grain_additions:
            self.assertEquals(grain_add.units, SI_UNITS)
        for hop_add in self.recipe.hop_additions:
            self.assertEquals(hop_add.units, SI_UNITS)

        recipe = self.recipe.change_units()
        self.assertEquals(recipe.units, IMPERIAL_UNITS)
        self.assertEquals(self.recipe.units, SI_UNITS)

    def test_get_total_points(self):
        out = self.recipe.get_total_points()
        self.assertEquals(round(out, 2), 1441.01)

    def test_get_total_points_lme(self):
        pale_lme = GrainAddition(pale,
                                 weight=4.75,
                                 grain_type=GRAIN_TYPE_LME,
                                 units=SI_UNITS)
        recipe = Recipe('lme',
                        grain_additions=[pale_lme],
                        hop_additions=self.hop_additions,
                        yeast=self.yeast,
                        units=SI_UNITS)

        out = recipe.get_total_points()
        self.assertEquals(round(out, 2), 1466.71)

    def test_get_total_points_dme(self):
        pale_dme = GrainAddition(pale,
                                 weight=3.80,
                                 grain_type=GRAIN_TYPE_DME,
                                 units=SI_UNITS)
        recipe = Recipe('dme',
                        grain_additions=[pale_dme],
                        hop_additions=self.hop_additions,
                        yeast=self.yeast,
                        units=SI_UNITS)

        out = recipe.get_total_points()
        self.assertEquals(round(out, 2), 1173.36)

    def test_get_original_gravity_units(self):
        out = self.recipe.get_original_gravity_units()
        self.assertEquals(round(out, 2), 76.13)

    def test_get_original_gravity(self):
        out = self.recipe.get_original_gravity()
        self.assertEquals(round(out, 3), 1.076)

    def test_get_boil_gravity_units(self):
        out = self.recipe.get_boil_gravity_units()
        self.assertEquals(round(out, 2), 54.38)

    def test_get_boil_gravity(self):
        out = self.recipe.get_boil_gravity()
        self.assertEquals(round(out, 3), 1.054)

    def test_get_final_gravity_units(self):
        out = self.recipe.get_final_gravity_units()
        self.assertEquals(round(out, 2), 19.03)

    def test_get_final_gravity(self):
        out = self.recipe.get_final_gravity()
        self.assertEquals(round(out, 3), 1.019)

    def test_brew_house_yield(self):
        vol_actual = 5.5 * LITER_PER_GAL
        bhy = self.recipe.get_brew_house_yield(SUCROSE_PLATO, vol_actual)
        self.assertEqual(round(bhy, 2), 0.66)

    def test_get_extract_weight(self):
        extract_weight = self.recipe.get_extract_weight()
        self.assertEquals(round(extract_weight, 2), 2.68)

    def test_get_percent_malt_bill(self):
        out = self.recipe.get_percent_malt_bill(self.grain_additions[0])
        self.assertEquals(round(out, 2), 0.95)
        out = self.recipe.get_percent_malt_bill(self.grain_additions[1])
        self.assertEquals(round(out, 2), 0.05)

    def test_get_grain_add_dry_weight(self):
        out = self.recipe.get_grain_add_dry_weight(self.grain_additions[0])
        self.assertEquals(round(out, 2), 2.66)
        out = self.recipe.get_grain_add_dry_weight(self.grain_additions[1])
        self.assertEquals(round(out, 2), 0.15)

    def test_get_grain_add_dry_weight_lme(self):
        pale_lme = GrainAddition(pale,
                                 weight=4.75,
                                 grain_type=GRAIN_TYPE_LME,
                                 units=SI_UNITS)
        recipe = Recipe('lme',
                        grain_additions=[pale_lme],
                        hop_additions=self.hop_additions,
                        yeast=self.yeast,
                        units=SI_UNITS)

        out = recipe.get_grain_add_dry_weight(pale_lme)
        self.assertEquals(round(out, 2), 3.80)

    def test_get_grain_add_dry_weight_dme(self):
        pale_dme = GrainAddition(pale,
                                 weight=3.80,
                                 grain_type=GRAIN_TYPE_DME,
                                 units=SI_UNITS)
        recipe = Recipe('dme',
                        grain_additions=[pale_dme],
                        hop_additions=self.hop_additions,
                        yeast=self.yeast,
                        units=SI_UNITS)

        out = recipe.get_grain_add_dry_weight(pale_dme)
        self.assertEquals(round(out, 2), 3.80)

    def test_get_total_dry_weight(self):
        out = self.recipe.get_total_dry_weight()
        self.assertEquals(round(out, 2), 2.81)

    def test_get_grain_add_cereal_weight(self):
        out = self.recipe.get_grain_add_cereal_weight(self.grain_additions[0])
        self.assertEquals(round(out, 2), 6.33)
        out = self.recipe.get_grain_add_cereal_weight(self.grain_additions[1])
        self.assertEquals(round(out, 2), 0.35)

    def test_get_grain_add_cereal_weight_lme(self):
        pale_lme = GrainAddition(pale,
                                 weight=4.75,
                                 grain_type=GRAIN_TYPE_LME,
                                 units=SI_UNITS)
        recipe = Recipe('lme',
                        grain_additions=[pale_lme],
                        hop_additions=self.hop_additions,
                        yeast=self.yeast,
                        units=SI_UNITS)

        out = recipe.get_grain_add_cereal_weight(pale_lme)
        self.assertEquals(round(out, 2), 9.05)

    def test_get_grain_add_cereal_weight_dme(self):
        pale_dme = GrainAddition(pale,
                                 weight=3.80,
                                 grain_type=GRAIN_TYPE_DME,
                                 units=SI_UNITS)
        recipe = Recipe('dme',
                        grain_additions=[pale_dme],
                        hop_additions=self.hop_additions,
                        yeast=self.yeast,
                        units=SI_UNITS)

        out = recipe.get_grain_add_cereal_weight(pale_dme)
        self.assertEquals(round(out, 2), 9.05)

    def test_get_total_grain_weight(self):
        total_grain_weight = self.recipe.get_total_grain_weight()
        self.assertEquals(round(total_grain_weight, 2), 6.69)

    def test_get_percent_ibus(self):
        percent_ibus = self.recipe.get_percent_ibus(self.hop_additions[0])
        self.assertEquals(round(percent_ibus, 2), 0.88)
        percent_ibus = self.recipe.get_percent_ibus(self.hop_additions[1])
        self.assertEquals(round(percent_ibus, 2), 0.12)

    def test_get_total_ibu(self):
        self.assertEqual(round(self.recipe.get_boil_gravity(), 3), 1.054)
        self.assertEqual(round(self.recipe.final_volume, 2), 18.93)
        total_ibu = self.recipe.get_total_ibu()
        self.assertEquals(round(total_ibu, 2), 33.03)

    def test_bu_to_gu(self):
        self.assertEqual(round(self.recipe.get_bu_to_gu(), 2), 0.61)

    def test_get_strike_temp(self):
        strike_temp = self.recipe.get_strike_temp(152.0, 60.0, 3.0 / 1.0)
        self.assertEquals(round(strike_temp, 2), 164.27)

    def test_get_mash_water_volume(self):
        mash_water_vol = self.recipe.get_mash_water_volume(3.0 / 1.0)
        self.assertEquals(round(mash_water_vol, 2), 8.42)

    def test_get_wort_color(self):
        wort_color = self.recipe.get_wort_color(self.grain_additions[0])
        self.assertEquals(round(wort_color, 2), 4.85)
        wort_color = self.recipe.get_wort_color(self.grain_additions[1])
        self.assertEquals(round(wort_color, 2), 3.26)

    def test_get_total_wort_color(self):
        total_wort_color = self.recipe.get_total_wort_color()
        self.assertEquals(round(total_wort_color, 2), 6.58)

    def test_get_total_wort_color_map(self):
        wort_map = self.recipe.get_total_wort_color_map()
        expected = {
            'srm': {'daniels': 10.1, 'morey': 6.6, 'mosher': 7.3},
            'ebc': {'daniels': 20.0, 'morey': 13.0, 'mosher': 14.4},
        }
        self.assertEquals(wort_map, expected)

    def test_to_json(self):
        self.assertEquals(self.recipe.units, SI_UNITS)
        out = self.recipe.to_json()
        expected = '{"data": {"abv_alternative": 7.98, "abv_standard": 7.49, "abw_alternative": 6.33, "abw_standard": 5.95, "boil_gravity": 1.054, "bu_to_gu": 0.6, "final_gravity": 1.019, "original_gravity": 1.076, "percent_brew_house_yield": 0.7, "total_ibu": 33.0, "total_wort_color_map": {"ebc": {"daniels": 20.0, "morey": 13.0, "mosher": 14.4}, "srm": {"daniels": 10.1, "morey": 6.6, "mosher": 7.3}}, "units": "metric"}, "final_volume": 18.93, "grains": [{"data": {"color": 2.0, "hwe": 308.78, "percent_malt_bill": 0.95, "ppg": 37.0, "short_name": "2-row", "working_yield": 0.56, "wort_color_ebc": 9.6, "wort_color_srm": 4.9}, "grain_type": "cereal", "name": "pale 2-row", "units": "metric", "weight": 6.33}, {"data": {"color": 20.0, "hwe": 292.09, "percent_malt_bill": 0.05, "ppg": 35.0, "short_name": "C20", "working_yield": 0.53, "wort_color_ebc": 6.4, "wort_color_srm": 3.3}, "grain_type": "cereal", "name": "crystal C20", "units": "metric", "weight": 0.35}], "hops": [{"boil_time": 60.0, "data": {"ibus": 29.2, "percent_alpha_acids": 0.14, "utilization": 0.24}, "hop_type": "pellet", "name": "centennial", "units": "metric", "utilization_cls": "Glenn Tinseth", "utilization_cls_kwargs": {"units": "metric"}, "weight": 16159.21}, {"boil_time": 5.0, "data": {"ibus": 3.9, "percent_alpha_acids": 0.07, "utilization": 0.05}, "hop_type": "pellet", "name": "cascade", "units": "metric", "utilization_cls": "Glenn Tinseth", "utilization_cls_kwargs": {"units": "metric"}, "weight": 21545.62}], "name": "Pale Ale", "start_volume": 26.5, "yeast": {"data": {"percent_attenuation": 0.75}, "name": "Wyeast 1056"}}'  # nopep8
        self.assertEquals(out, expected)

    def test_format(self):
        self.assertEquals(self.recipe.units, SI_UNITS)
        out = self.recipe.format()
        expected = textwrap.dedent("""\
            Pale Ale
            ===================================

            Brew House Yield:   0.70 %
            Start Volume:       26.5
            Final Volume:       18.9

            Original Gravity:   1.076
            Boil Gravity:       1.054
            Final Gravity:      1.019

            ABV / ABW Standard: 7.49 % / 5.95 %
            ABV / ABW Alt:      7.98 % / 6.33 %

            IBU:                33.0 ibu
            BU/GU:              0.6

            Morey   (SRM/EBC):  6.6 degL / 13.0
            Daneils (SRM/EBC):  10.1 degL / 20.0
            Mosher  (SRM/EBC):  7.3 degL / 14.4

            Grains
            ===================================

            pale 2-row Addition
            -----------------------------------
            Grain Type:        cereal
            Weight:            6.33 kg
            Percent Malt Bill: 0.95 %
            Working Yield:     0.56 %
            SRM/EBC:           4.9 degL / 9.6

            crystal C20 Addition
            -----------------------------------
            Grain Type:        cereal
            Weight:            0.35 kg
            Percent Malt Bill: 0.05 %
            Working Yield:     0.53 %
            SRM/EBC:           3.3 degL / 6.4

            Hops
            ===================================

            centennial Addition
            -----------------------------------
            Hop Type:     pellet
            AA %:         0.14 %
            Weight:       16159.21 mg
            Boil Time:    60.0 min
            IBUs:         29.2
            Utilization:  0.24 %

            cascade Addition
            -----------------------------------
            Hop Type:     pellet
            AA %:         0.07 %
            Weight:       21545.62 mg
            Boil Time:    5.0 min
            IBUs:         3.9
            Utilization:  0.05 %

            Yeast
            ===================================

            Wyeast 1056 Yeast
            -----------------------------------
            Attenuation:  0.75 %""")
        self.assertEquals(out, expected)

    def test_validate(self):
        data = self.recipe.to_dict()
        Recipe.validate(data)



#! /usr/bin/env python

"""
Brewing Classic Styles: 80 Winning Recipes Anyone Can Brew
by Jamil Zainasheff and John J. Palmer

Biere de l'Inde

Used by permission of Brewers Publications (2007). All rights reserved.

You can purchase the book here:

- http://www.brewerspublications.com/books/brewing-classic-styles-80-winning-recipes-anyone-can-brew/

Original Stats:

OG:      1.062 (15.2P)
FG:      1.015 ( 4.0P)
ADF:     74%
IBU:     50
Color:   11 SRM (22 EBC)
Alcohol: 6.2% ABV (4.8% ABW)
Boil:    60 min
Pre-Boil Volume:  7 Gal (26.5L)
Pre-Boil Gravity: 1.053 (13.0P)
"""  # nopep8

import os

from brew.parsers import JSONDataLoader
from brew.parsers import parse_recipe


def main():

    recipe = {
        'name': "Biere de l'Inde (Extract)",
        'start_volume': 7.0,
        'final_volume': 6.0,
        'grains': [
            {'name': 'English Pale Ale Liquid Extract',
             'data': {
                 'color': 3.5,
                 'ppg': 37,
             },
             'weight': 8.7,
             'grain_type': 'lme'},
            {'name': 'wheat liquid extract',
             'data': {
                 'color': 4.0,
                 'ppg': 37,
             },
             'weight': 0.5,
             'grain_type': 'lme'},
            {'name': 'biscuit malt',
             'data': {
                 'color': 25.0,
             },
             'weight': 0.5,
             'grain_type': 'specialty'},
            {'name': 'caramel crystal malt 40l',
             'weight': 0.5,
             'grain_type': 'specialty'},
            {'name': 'caramel crystal malt 120l',
             'weight': 0.375,
             'grain_type': 'specialty'},
        ],
        'hops': [
            {'name': 'challenger',
             'data': {
                 'percent_alpha_acids': 0.08,
             },
             'weight': 1.43,
             'boil_time': 60.0},
            {'name': 'fuggle',
             'data': {
                 'percent_alpha_acids': 0.05,
             },
             'weight': 1.5,
             'boil_time': 10.0},
            {'name': 'east kent golding',
             'data': {
                 'percent_alpha_acids': 0.05,
             },
             'weight': 1.5,
             'boil_time': 0.0},
        ],
        'yeast': {
            'name': 'Wyeast 1028',
            'data': {
                'percent_attenuation': 0.74,
            },
        },
        'data': {
            'percent_brew_house_yield': 0.70,
            'units': 'imperial',
        },
    }

    data_dir = os.path.abspath(os.path.join(os.getcwd(), 'data/'))
    loader = JSONDataLoader(data_dir)
    beer = parse_recipe(recipe, loader)
    print(beer.format())


if __name__ == "__main__":
    main()
